@@ -242,11 +242,9 @@ def tokenize_numeric_col(series, nparts=2, col_zfill=2):
         raise ValueError(
             f"Partition size {nparts} is greater than the value length {max_len}. Consider reducing the number of partitions..."
         )
+    mx = series.map(len).max()
 
-    tr = series.map(
-        lambda x: [x[i : i + nparts] for i in range(0, len(x), nparts)]
-    ).apply(pd.Series)
-
+    tr = pd.concat([series.str[i : i + nparts] for i in range(0, mx, nparts)], axis=1)
     tr.columns = encode_partition_numeric_col(col, tr, col_zfill)
 
     return tr
@@ -347,7 +345,7 @@ def process_data(
     col_idx = {col: f"{str(i).zfill(num_cols)}" for i, col in enumerate(df.columns)}
 
     # Create a dataframe that will hold the processed data
-    processed_df = pd.DataFrame()
+    processed_series = []
 
     # Process numerical data
     numeric_cols = df.select_dtypes(include=np.number).columns
@@ -364,7 +362,8 @@ def process_data(
             transform_data=col_transform_data.get(c),
         )
         col_transform_data[c] = transform_data
-        processed_df[col_name] = series
+        series.name = col_name
+        processed_series.append(series)
 
     # Process datetime data
     datetime_cols = df.select_dtypes(include="datetime").columns
@@ -375,8 +374,10 @@ def process_data(
             df[c], transform_data=col_transform_data.get(c)
         )
         col_transform_data[c] = transform_data
+        series.name = col_name
+        processed_series.append(series)
 
-        processed_df[col_name] = series
+    processed_df = pd.concat(processed_series, axis=1)
 
     if not processed_df.empty:
         # Combine the processed numeric and datetime data.
