@@ -4,6 +4,7 @@ algorithms used for tabular and relational data generation.
 import logging
 from typing import Any, Dict, List, Optional, Union
 
+import datasets
 import numpy as np
 import pandas as pd
 import torch
@@ -166,6 +167,7 @@ class REaLSampler:
     ) -> Union[torch.tensor, np.ndarray]:
         # This leverages the generic interface of HuggingFace transformer models' `.generate` method.
         # Refer to the transformers documentation for valid arguments to `generate_kwargs`.
+        self.model.eval()
 
         if constrain_tokens_gen:
             generate_kwargs["prefix_allowed_tokens_fn"] = self._prefix_allowed_tokens_fn
@@ -181,6 +183,15 @@ class REaLSampler:
         else:
             generate_kwargs["suppress_tokens"].append(RMASK_ID)
 
+        if "bos_token_id" not in generate_kwargs:
+            generate_kwargs["bos_token_id"] = self.vocab["token2id"][SpecialTokens.BOS]
+
+        if "pad_token_id" not in generate_kwargs:
+            generate_kwargs["pad_token_id"] = self.vocab["token2id"][SpecialTokens.PAD]
+
+        if "eos_token_id" not in generate_kwargs:
+            generate_kwargs["eos_token_id"] = self.vocab["token2id"][SpecialTokens.EOS]
+
         _samples = self.model.generate(**generate_kwargs)
 
         if as_numpy:
@@ -437,6 +448,31 @@ class TabularSampler(REaLSampler):
 
         self.output_vocab = self.vocab
 
+    @staticmethod
+    def sampler_from_model(rtf_model, device: str = "cuda"):
+        device = torch.device(device)
+
+        assert rtf_model.tabular_max_length is not None
+        assert rtf_model.tabular_col_size is not None
+        assert rtf_model.col_transform_data is not None
+
+        return TabularSampler(
+            model_type=rtf_model.model_type,
+            model=rtf_model.model,
+            vocab=rtf_model.vocab,
+            processed_columns=rtf_model.processed_columns,
+            max_length=rtf_model.tabular_max_length,
+            col_size=rtf_model.tabular_col_size,
+            col_idx_ids=rtf_model.col_idx_ids,
+            columns=rtf_model.columns,
+            datetime_columns=rtf_model.datetime_columns,
+            column_dtypes=rtf_model.column_dtypes,
+            drop_na_cols=rtf_model.drop_na_cols,
+            col_transform_data=rtf_model.col_transform_data,
+            random_state=rtf_model.random_state,
+            device=device,
+        )
+
     def _prefix_allowed_tokens_fn(self, batch_id, input_ids) -> List:
         # https://huggingface.co/docs/transformers/v4.24.0/en/main_classes/text_generation#transformers.generation_utils.GenerationMixin.generate.prefix_allowed_tokens_fn
         # For the tabular data, len(input_ids) == 1 -> [BOS]
@@ -566,6 +602,84 @@ class TabularSampler(REaLSampler):
 
         return synth_df
 
+    def predict(
+        self,
+        data: pd.DataFrame,
+        target_col: str,
+        target_pos_val: Any = None,
+        batch: int = 32,
+        obs_sample: int = 30,
+        fillunk: bool = True,
+        device: str = "cuda",
+        disable_progress_bar: bool = True,
+        **generate_kwargs,
+    ) -> pd.Series:
+        """
+        fillunk: Fill unknown tokens with the mode of the batch.
+        target_pos_val: Categorical value for the positive target. This is produces a
+         one-to-many prediction relative to `target_pos_val` for targets that are multi-categorical.
+        """
+        device = torch.device(device)
+
+        if self.model.device != device:
+            self.model = self.model.to(device)
+
+        self.model.eval()
+
+        preds = []
+        unk_id = self.vocab["token2id"][SpecialTokens.UNK]
+
+        if target_col and target_col in data.columns:
+            data = data.drop(target_col, axis=1)
+
+        if disable_progress_bar:
+            datasets.utils.disable_progress_bar()
+
+        for i in range(0, len(data), batch):
+            seed_data = self._process_seed_input(data.iloc[i : i + batch])
+            if fillunk:
+                mode = seed_data.mode(dim=0).values
+                seed_data[seed_data == unk_id] = torch.tile(mode, (len(seed_data), 1))[
+                    seed_data == unk_id
+                ]
+
+            sample_outputs = self._generate(
+                device=device,
+                do_sample=True,
+                num_return_sequences=obs_sample,
+                input_ids=seed_data.to(device),
+                max_length=self.max_length,
+                suppress_tokens=[unk_id],
+                **generate_kwargs,
+            )
+
+            synth_sample = self._processes_sample(
+                sample_outputs=sample_outputs,
+                vocab=self.vocab,
+                validator=None,
+            )
+            # Reset the index so that we are sure that
+            # the index is monotonically increasing.
+            # There could be instances where some generation
+            # problems arise for some records, we don't
+            # handle that here for now.
+            synth_sample.reset_index(drop=True, inplace=True)
+
+            preds.extend(
+                synth_sample.groupby(synth_sample.index // obs_sample)[
+                    target_col
+                ].apply(
+                    lambda x: (target_pos_val == x).mean()
+                    if target_pos_val is not None
+                    else x.mean()
+                )
+            )
+
+        if disable_progress_bar:
+            datasets.utils.enable_progress_bar()
+
+        return pd.Series(preds, index=data.index)
+
 
 class RelationalSampler(REaLSampler):
     """Sampler class for relational data generation."""
@@ -608,6 +722,33 @@ class RelationalSampler(REaLSampler):
         self.output_vocab = self.vocab["decoder"]
         self.in_col_transform_data = in_col_transform_data
 
+    @staticmethod
+    def sampler_from_model(rtf_model, device: str = "cuda"):
+        device = torch.device(device)
+
+        assert rtf_model.relational_max_length is not None
+        assert rtf_model.relational_col_size is not None
+        assert rtf_model.col_transform_data is not None
+        assert rtf_model.in_col_transform_data is not None
+
+        return RelationalSampler(
+            model_type=rtf_model.model_type,
+            model=rtf_model.model,
+            vocab=rtf_model.vocab,
+            processed_columns=rtf_model.processed_columns,
+            max_length=rtf_model.relational_max_length,
+            col_size=rtf_model.relational_col_size,
+            col_idx_ids=rtf_model.col_idx_ids,
+            columns=rtf_model.columns,
+            datetime_columns=rtf_model.datetime_columns,
+            column_dtypes=rtf_model.column_dtypes,
+            drop_na_cols=rtf_model.drop_na_cols,
+            col_transform_data=rtf_model.col_transform_data,
+            in_col_transform_data=rtf_model.in_col_transform_data,
+            random_state=rtf_model.random_state,
+            device=device,
+        )
+
     def sample_relational(
         self,
         input_unique_ids: Union[pd.Series, List],
